from pyalsi.hardware.disks.disk import Disk, DiskGroup
from nose.plugins.attrib import attr

from pyalsi.tests.base_test import BaseTest
from pyalsi.utils.strings import Colors


@attr('small', 'hardware', 'unit', 'disk')
class TestDiskUnit(BaseTest):

    def test_diskgroup(self):
        Colors.colors['c2'] = ''
        haz_root = False
        group = DiskGroup()
        self.assertIsInstance(group, DiskGroup)
        self.assertIsInstance(group.disks, list)
        for disk in group.disks:
            self.assertIsInstance(disk, Disk)
            if disk.name == 'Root':
                haz_root = True
            self.assertIn('G', disk.to_info_string())
        self.assertTrue(haz_root)



import psutil

from pyalsi.utils.types import Bytes
from pyalsi.utils.strings import colorize_usage, colorize_percent, Colors


class Disk(object):
    def __init__(self, disk):
        name = disk.mountpoint.split('/')[-1]
        self.usage = self.Usage(psutil.disk_usage(disk.mountpoint))
        self.name = name.capitalize() if name != "" else "Root"  # Type: str
        self.fstype = disk.fstype

    def to_info_string(self):
        return ("{c2}{}: {} ({}) ({})".format(
            self.name,
            colorize_usage(self.usage.used.to_gigabytes(),
                           self.usage.total.to_gigabytes(),
                           self.usage.percent, "G"),
            colorize_percent(self.usage.percent, "%"),
            self.fstype, **Colors.colors))

    class Usage(object):
        def __init__(self, usage):
            self.used = Bytes(usage.used)
            self.total = Bytes(usage.total)
            self.percent = int(usage.percent)


class DiskGroup(object):
    def __init__(self):
        self.disks = [Disk(disk) for disk in psutil.disk_partitions()]



from nose.plugins.attrib import attr

from pyalsi.tests.base_test import BaseTest
from pyalsi.system.system import System
from pyalsi.logos import logos

@attr('small', 'sanity', 'unit')
class TestUtilsUnit(BaseTest):

    def test_systems_in_logos(self):
        for distro in System().distro_subclass_map.keys():
            self.assertIn(distro, logos.keys())



import os


class VgaCard(object):
    pass


class Pci(object):
    @staticmethod
    def get_vga_devices():
        out = os.popen("lspci | grep VGA").read().splitlines()
        cards = {}
        for line in out:
            line = line.split(":")[-1]
            if line in cards.keys():
                count = cards[line]
                cards.pop(line)
                cards[line] = count + 1
            else:
                cards[line] = 1
        output = []
        if len(cards) > 0:
            for k, v in cards.iteritems():
                output.append("{}{}".format(k, ("(x{})".format(v)) if v > 1 else ""))
        else:
            return False
        return output



from cpuinfo import cpuinfo


class Cpu(object):
    def __init__(self):
        self.count = None
        self.brand = None
        self.__dict__.update(cpuinfo.get_cpu_info_from_proc_cpuinfo())

    def to_info_string(self):
        fmt = "{} cores" if self.count > 1 else "{} core"
        return "{} ({})".format(self.brand, fmt.format(self.count))



"""
PROCESS_NAME  :  WINDOW_MANAGER_NAME
"""

window_manager_definitions = {
    '2wm': '2wm',
    '9wm': '9wm',
    'afterstep': 'AfterStep',
    'antiwm': 'AntiWM',
    'awesome': 'Awesome',
    'barewm': 'Bare WM',
    'beryl': 'Beryl',
    'blackbox': 'Blackbox',
    'clfswm': 'CLFS WM',
    "cinnamon": 'Cinnamon',
    'compiz': 'Compiz',
    'ctwm': 'CTWM',
    'cwm': 'CWM',
    'dswm': 'DSWM',
    'dwm': 'DWM',
    'echinus': 'Echinus',
    'enlightenment': 'Enlightenment',
    'euclid': 'Euclid',
    'evil-wm': 'EvilWM',
    'evilpoison': 'EviL Poison',
    'fluxbox': 'Fluxbox',
    'flwm': 'FLWM',
    'fvwm': 'FVWM',
    'hackedbox': 'Hackedbox',
    'hildon-desktop': 'Matchbox',
    'i3': 'i3',
    'icewm': 'IceWM',
    'ion-3': 'Ion3',
    'ion3': 'Ion3',
    'jwm': 'JWM',
    'karmen': 'Karmen',
    'kwin': 'Kwin',
    'kodi-standalone': 'Kodi',
    'larswm': 'Lars WM',
    'lewm': 'LEWM',
    'lwm': 'LWM',
    'matwm2': 'matWM2',
    'metacity': 'Metacity',
    'musca': 'Musca',
    'notion': 'Notion',
    'openbox': 'Openbox',
    'openbox-gnome-session': 'Openbox (GNOME session)',
    'openbox-kde-session': 'Openbox (KDE session)',
    'pawm': 'PAWM',
    'pekwm': 'PekWM',
    'qlwm': 'qLWM',
    'qtile': 'qTile',
    'quarkwm': 'QuarkWM',
    'ratpoison': 'ratpoison',
    'sawfish': 'Sawfish',
    'scrot-wm': 'ScrotWM',
    'scrotwm': 'ScrotWM',
    'sithwm': 'SithWM',
    'stumpwm': 'StumpWM',
    'subtle': 'Subtle',
    'swm': 'SWM',
    'tinywm': 'TinyWM',
    'tritium': 'Tritium',
    'twm': 'twm',
    'velox': 'Velox',
    'vtwm': 'VtWM',
    'vwm': 'VWM',
    'w9wm': 'w9wm',
    'weewm': 'WeeWM',
    'windowmaker': 'WindowMaker',
    'windwm': 'WindWM',
    'wmaker': 'WindowMaker',
    'wmfs': 'Wmfs',
    'WMFS': 'WMFS',
    'wmii': 'wmii',
    'wmii36': 'wmii36',
    'xmonad': 'xmonad',
    'yeahwm': 'YeahWM',
}



import psutil
from pyalsi.utils import types


class Ram(object):
    def __init__(self):
        self.percent = None
        self.total = None
        self.used = None
        self.__dict__.update(psutil.virtual_memory().__dict__)

    def get_total(self):
        return types.Bytes(self.total)

    def get_used(self):
        return types.Bytes(self.used)



from pyalsi.hardware.vga.vga import VgaCard, Pci
from nose.plugins.attrib import attr

from pyalsi.tests.base_test import BaseTest
from pyalsi.utils.strings import Colors
from pyalsi.tests.mock_classes.os_popen import MockOsPopen
from mock import patch


@attr('small', 'hardware', 'unit', 'pci')
class TestVgaUnit(BaseTest):

    def test_vga_vanilla(self):
        pci_devices = Pci.get_vga_devices()
        self.assertIsInstance(pci_devices, list)

    @patch('os.popen', MockOsPopen)
    def test_vga_dual_mocked(self):
        self.assertIn('x2', Pci.get_vga_devices()[0])





from nose.plugins.attrib import attr

from pyalsi.tests.base_test import BaseTest
from pyalsi.utils.types import Bytes, Gigabyte, Megabyte


@attr('small', 'hardware', 'unit', 'disk')
class TestUtilsUnit(BaseTest):

    def test_types(self):
        self.assertEqual(Megabyte(1024).to_gigabytes(), 1)
        self.assertEqual(Megabyte(1).to_bytes(), 1048576)
        self.assertEqual(Bytes(1048576).to_megabytes(), 1)
        self.assertEqual(Bytes(1073741824).to_gigabytes(), 1)
        self.assertEqual(Gigabyte(1).to_bytes(), 1073741824)
        self.assertEqual(Gigabyte(1).to_megabytes(), 1024)



from mock import Mock


class MockOsPopen(Mock):
    def read(self, *args, **kwargs):
        return MockOsPopenSplitlines


class MockOsPopenSplitlines(object):
    @staticmethod
    def splitlines(*args, **kwargs):
        return ['00:02.0 VGA compatible controller: Intel Corporation Iris Graphics 6100 (rev 09)',
                '00:02.0 VGA compatible controller: Intel Corporation Iris Graphics 6100 (rev 09)']






class Bytes(int):
    def to_megabytes(self):
        return Megabyte(int(self.real / 1024 ** 2))

    def to_gigabytes(self):
        return Gigabyte(int(self.real / 1024 ** 3))


class Megabyte(int):
    def to_bytes(self):
        return Bytes(self.real * 1024 ** 2)

    def to_gigabytes(self):
        return Gigabyte(int(self.real / 1024 ** 1))


class Gigabyte(int):
    def to_bytes(self):
        return Bytes(self.real * 1024 ** 3)

    def to_megabytes(self):
        return Megabyte(self.real * 1024 ** 1)



from pyalsi.colors import bold, normal


class Colors(object):
    colors = {"low": bold["green"],
              "med": bold["yellow"],
              "high": bold["red"],
              "red": normal["red"]
              }

    def set(self, k, v):
        self.colors[k] = v


def colorize(heading, value):
    """
    wrapper to colorize info lines
    :param heading: line heading (eg. "OS")
    :param value: line value (eg. "Arch Linux")
    :return: string (eg. "{c2}OS: {c1}Arch Linux")
    """
    return "{c2}" + heading + ": {c1}" + str(value)


def colorize_usage(use, total, percent, unit):
    """
    colorizes the passed string based on the percentage passed in
    :param use: output variable
    :param total: output variable
    :param percent: used to calculate the appropriate color
    :param unit: essentially a suffix to use and total (eg. "G | M")
    :return: string (eg. "{low}43G{c1} / 87G")
    """
    level = '{low}' if percent <= 50 else '{med}' if percent < 80 else '{high}'
    return level + str(use) + unit + "{c1} / " + str(total) + unit


def colorize_percent(value, suffix=""):
    """
    as above but only takes one output variable
    :param value: the output variable
    :param suffix: string to put after value (eg. "%)
    :return: string (eg. "{low}15%{c1}")
    """
    level = '{low}' if value <= 50 else '{med}' if value < 80 else '{high}'
    return level + str(value) + suffix + "{c1}"



#! /usr/bin/python2

import click
import platform

import sys

from pyalsi.logos import logos
from colors import normal, bold
from pyalsi.hardware.disks.disk import DiskGroup
from pyalsi.hardware.vga.vga import VgaCard, Pci
from pyalsi.system.system import System
from window_managers import window_manager_definitions
from pyalsi.hardware.cpu.cpu import Cpu
from pyalsi.hardware.ram.ram import Ram
from pyalsi.utils.strings import colorize, colorize_usage, colorize_percent, Colors

DEFAULT_DISTRO_LOGO = {'Arch Linux': 'Archey',
                       'Debian': 'Below',
                       'Fedora': 'Screenfetch',
                       'Apricity OS': 'Below',
                       'Ubuntu': 'Archey'}

cpu, ram, system, vga, disks = Cpu(), Ram(), System(), Pci(), DiskGroup()
colors = Colors()


def set_color_one(ctx, p, n):
    if n not in normal:
        invalid_colour_warning()
    colors.set('c1', normal[n])


def set_color_two(ctx, p, b):
    if b not in bold:
        invalid_colour_warning()
    colors.set('c2', bold[b])


def invalid_colour_warning():
    raise click.UsageError('color must be one of {}'.format(', '.join(normal)))


def validate_logo(ctx, p, logo):
    if logo is not None:
        if logo not in logos[system.distro]:
            valid_logos = "Please pick either: {}".format(", ".join(logos[system.distro].keys()))
            if logo != 'help':
                valid_logos = "Invalid logo. {}".format(valid_logos)
            raise click.UsageError(valid_logos)
    else:
        logo = DEFAULT_DISTRO_LOGO[system.friendly_distro]
    return logo


def set_distro(ctx, p, distro):
    if distro is not None:
        if distro not in logos.keys():
            raise click.UsageError('distro must be one of ({})'.format('|'.join(logos.keys())))
        system.distro = distro
        system.friendly_distro = system.distro_subclass_map[distro].friendly_distro
    return distro

@click.command()
@click.option('-n', '--normal-colour', default="white", callback=set_color_one)
@click.option('-b', '--bold-colour', default="dgrey", callback=set_color_two)
@click.option('-l', '--info-below', is_flag=True)
@click.option('-d', '--distro', help="choose: " + ", ".join(logos.keys()),
              callback=set_distro)
@click.option('--logo', help="type 'pyAlsi --logo help' to see valid logos for your distro",
              default=None, callback=validate_logo)
def cli(normal_colour, bold_colour, info_below, distro, logo):
    if logo == 'Below':
        info_below = True

    a = system.get_last_login()
    info = [colorize("OS", "{} {}".format(system.friendly_distro, platform.machine())),
            colorize("Hostname", platform.node()),
            colorize("Last Login From", '{} At {}'.format(a['ip'], a['at'])),
            colorize("Uptime", system.get_uptime()),
            colorize("Kernel", platform.release()),
            colorize("Shell", system.shell),
            colorize("Packages", system.count_packages()),
            colorize("Window Manager", system.get_window_manager()),
            colorize("RAM", "{} ({})".format(
                colorize_usage(ram.get_used().to_megabytes(), ram.get_total().to_megabytes(), ram.percent, "M"),
                colorize_percent(ram.percent, "%"))),
            colorize("CPU", cpu.to_info_string())
            ]

    info.extend(colorize("VGA Cards", card) for card in vga.get_vga_devices())
    info.extend(colorize(key, value) for key, value in system.get_package_stats().iteritems())
    info.extend(disk.to_info_string() for disk in disks.disks)

    if info_below:
        click.echo("\n".join([line.format(**colors.colors) for line in logos[system.distro][logo].splitlines()]))
        click.echo("\n\n")
        click.echo("\n".join(["   " + line.format(**colors.colors) for line in info]))
    else:
        for i, line in enumerate((logos[system.distro][logo]).splitlines()):
            click.echo("{}".format(line + (info[i] if (i < len(info)) else "")).format(**colors.colors))
    click.echo('\x1b[0m')  # return terminal colour to normal


# compatibility for debugging in pyCharm.
if __name__ == '__main__':
    print sys.argv
    cli(sys.argv[1:])



from pyalsi.tests.base_test import BaseTest
from pyalsi import cli
from pyalsi.logos import logos
from pyalsi.colors import normal, bold
from nose.plugins.attrib import attr
from pyalsi.system.system import System

@attr('common')
class TestCommonCommandFlags(BaseTest):

    @attr('small')
    def test_unflagged_functionality(self):
        result = self.runner.invoke(cli)
        self.assertEqual(result.exit_code, 0)
        self.assertIn('OS', result.output)

    @attr('medium')
    def test_distro_flag_functionality(self):
        for distro in System().distro_subclass_map.keys():
            result = self.runner.invoke(cli, ['-d', distro])
            self.assertIn(distro, result.output)
            self.assertEqual(result.exit_code, 0)

    @attr('medium')
    def test_logo_flag_functionality(self):
        for distro in System().distro_subclass_map.keys():
            for logo in logos[distro]:
                result = self.runner.invoke(cli, ['-d', distro, '--logo', logo])
                self.assertIn(distro, result.output)
                self.assertEqual(result.exit_code, 0)

    @attr('large')
    def test_colour_flag_functionality(self):
        for n in normal:
            for b in bold:
                result = self.runner.invoke(cli, ['-n', n, '-b', b])
                self.assertEqual(result.exit_code, 0)

    @attr('medium')
    def test_info_below_flag_functionality(self):
        for distro in System().distro_subclass_map.keys():
            result = self.runner.invoke(cli, ['-d', distro, '-l'])
            self.assertNotIn('\tOS', result.output)



from setuptools import setup, find_packages

setup(
    name='pyalsi',
    version='0.4.4.4',
    packages=find_packages(),
    install_requires=['psutil', 'click', 'py-cpuinfo'],
    license='GPL2',
    author='chestm007',
    author_email='chestm007@hotmail.com',
    url='https://github.com/chestm007/pyalsi',
    download_url='https://github.com/chestm007/pyalsi/mypackage/archive/0.4.3.tar.gz',
    description='python rewrite of alsi (Arch Linux System Information) to support multiple systems',
    entry_points="""
        [console_scripts]
        pyalsi=pyalsi.__init__:cli
    """,
    requires=[
        'nose',
        'mock'
    ]
)



import os
import math
from datetime import timedelta
from pyalsi.window_managers import window_manager_definitions


class System(object):
    distro = "unknown"
    friendly_distro = 'Unknown'
    distro_subclass_map = {}

    def __init__(self):
        self.distro = self.get_distro()
        for sub in System.__subclasses__():
            self.distro_subclass_map[sub.distro] = sub
            if self.distro == sub.distro:
                self.__class__ = sub
            for s in sub.__subclasses__():
                self.distro_subclass_map[s.distro] = s
                if self.distro == s.distro:
                    self.__class__ = s

        self.shell = os.readlink('/proc/%d/exe' % os.getppid())

    def get_distro(self):
        try:
            for line in os.popen("cat /etc/*-release").read().splitlines():
                if line.startswith('NAME='):
                    return line.split('=')[1].strip('"')
        except IOError:
            return "Unknown"

    @staticmethod
    def get_uptime():
        with open('/proc/uptime', 'r') as f:
            uptime_seconds = float(f.readline().split()[0])
            return str(timedelta(seconds=math.ceil(uptime_seconds)))

    def get_window_manager(self):
        for proc in self.get_processes():
            wm = window_manager_definitions.get(proc)
            if wm:
                return wm

    @staticmethod
    def get_last_login():
        out = os.popen("last $USER -i | grep -E -v 'logged'").read()
        for o in out.splitlines():
            o = o.split()
            if len(o) > 1:
                if not o[-1] == 'in' and not o[0] == 'wtmp':
                    output_dict = {'name': o[0],
                                   'tty': o[1],
                                   'ip': o[2],
                                   'at': '{} {} {} {}:{}'.format(
                                       o[3], o[4], o[5], o[6], o[7].strip(':-'))}
                    return output_dict

    @staticmethod
    def get_processes():
        processes = os.popen('ps -A').read().splitlines()
        processes = [line.split()[3] for line in processes]
        return processes

    def get_package_stats(self):
        return {'Pending Updates': self._get_package_stats()}

    def _get_package_stats(self):
        return 'no data'

    def count_packages(self):
        return self._count_packages()

    def _count_packages(self):
        return 'no data'


class ArchLinuxSystem(System):
    distro = 'Arch'
    friendly_distro = 'Arch Linux'

    def _count_packages(self):
        return PackageManager.Pacman.count_packages()

    def _get_package_stats(self):
        return PackageManager.Pacman.get_package_stats()


class ApricitySystem(ArchLinuxSystem):
    distro = 'Apricity OS'
    friendly_distro = 'Apricity OS'


class DebianSystem(System):
    distro = 'Debian'
    friendly_distro = distro

    def _count_packages(self):
        return PackageManager.Dpkg.count_packages()


class UbuntuSystem(DebianSystem):
    distro = 'Ubuntu'
    friendly_distro = distro

    def _get_package_stats(self):
        return PackageManager.Apt.get_package_stats()


class FedoraSystem(System):
    distro = 'Fedora'
    friendly_distro = distro

    def _count_packages(self):
        return PackageManager.Dpkg.count_packages()


class PackageManager(object):
    class Dpkg(object):
        @staticmethod
        def count_packages():
            for result in os.popen('dpkg -l |grep ^ii | wc -l').read().splitlines():
                if result:
                    return result

    class Apt(object):
        @staticmethod
        def get_package_stats():
            apt_output = os.popen('/usr/lib/update-notifier/apt-check --human-readable').read().splitlines()
            return apt_output[0].split()[0]

    class Pacman(object):
        @staticmethod
        def count_packages():
            return len([name for name in os.listdir('/var/lib/pacman/local')])

        @staticmethod
        def get_package_stats():
            pacman_output = os.popen('pacman -Qu').read().splitlines()
            return str(len(pacman_output))
